# Use PIO to drive a "neopixel" (WS2812) stick of LEDS
# CC0 code from www.tinkerfarm.net

# circuit:
# Insert the stick across 3 rows (note only 1 GND has a pin)
# Neopixels require 5 volts so jumper physical pin 40
#    (VBUS) to VCC on the stick
# Jumper ground rail to stick GND
# Jumper GP15 (physical pin 20) to stick IN

# import modules
import array, time
from machine import Pin
# rp2 will allow us to use programable input/output (PIO) pins
import rp2
from rp2 import PIO, StateMachine, asm_pio

# Configure the number of LEDs to 8.
NUM_LEDS = 8

# create the assembly code to run on the PIO
# the asm dectorator sets up the processor
@asm_pio(sideset_init=PIO.OUT_LOW, out_shiftdir=PIO.SHIFT_LEFT,
autopull=True, pull_thresh=24)
# this function gives the PIO its instructions in assembly
def ws2812():
    T1 = 2
    T2 = 5
    T3 = 3
    label("bitloop")
    out(x, 1) .side(0) [T3 - 1]
    jmp(not_x, "do_zero") .side(1) [T1 - 1]
    jmp("bitloop") .side(1) [T2 - 1]
    label("do_zero")
    nop() .side(0) [T2 - 1]
 
# Create the StateMachine with the ws2812 program,
# make it program 0, set the frequency and output on GP15 (physical pin 20).
sm = StateMachine(0, ws2812, freq=8000000, sideset_base=Pin(15))

# Start the StateMachine, it will wait for data on its FIFO buffer.
sm.active(1)

# We will be feeding binary numbers to the PIO.
# The PIO wants a 24 bit number with 8 bits for each color.
# First create an array sized to our stick of LEDs.
# I tells micropython we're using 32 bit numbers.
ar = array.array("I", [0 for _ in range(NUM_LEDS)])

# loop forever

while True:
    print("blue")
    # we do this 255 times to
    # ramp given color from off to brightest
    for j in range(0, 255):
        # now for each led in the stick
        for i in range(NUM_LEDS):
            # set the element in the array to brightness
            ar[i] = j
            # delete the last 8 bits going to the PIO
            # and send it
            sm.put(ar,8)
            time.sleep_ms(1)

    print("red")
    for j in range(0, 255):
        for i in range(NUM_LEDS):
            # shift the bits 8 places to be in red slots
            ar[i] = j<<8
            sm.put(ar,8)
            time.sleep_ms(1)

    print("green")
    for j in range(0, 255):
        for i in range(NUM_LEDS):
            # shift the bits 16 places to be in green slots
            ar[i] = j<<16
            sm.put(ar,8)
            time.sleep_ms(1)

    print("slow chase")
    for i in range(4 * NUM_LEDS):
        for j in range(NUM_LEDS):
            r = j * 100 // (NUM_LEDS - 1)
            b = 100 - j * 100 // (NUM_LEDS - 1)
            if j != i % NUM_LEDS:
                r >>= 3
                b >>= 3
            ar[j] = r << 16 | b
        sm.put(ar, 8)
        time.sleep_ms(100)

    print("sparkle chase")  
    for i in range(24):
        for j in range(NUM_LEDS):
            ar[j] >>= 1
        sm.put(ar, 8)
        time.sleep_ms(100)
        
  
  